# Copyright (c) 2005 Infrae. All rights reserved.
# See also LICENSE.txt

from AccessControl import ClassSecurityInfo
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from Globals import InitializeClass

# Zope 3
from zope.interface.interfaces import IInterface
from zope.interface import directlyProvides, directlyProvidedBy

# Silva
from Products.Silva import SilvaPermissions
from Products.Silva.VersionedContent import CatalogedVersionedContent
from Products.Silva.Version import CatalogedVersion
from Products.Silva import mangle

# interfaces
from Products.Silva.interfaces import IVersionedContent, IVersion
from zope.interface import implements

# SilvaFlexibleXML
from Products.SilvaFlexibleXML import xmlsource
from Products.SilvaFlexibleXML import interfaces
from Products.SilvaFlexibleXML import error

class XMLDocumentBase(CatalogedVersionedContent):
    security = ClassSecurityInfo()

    __implements__ = IVersionedContent

    implements(interfaces.IXMLDocument)
    
    # A hackish way to get a Silva tab in between the standard ZMI tabs
    inherited_manage_options = CatalogedVersionedContent.manage_options
    manage_options=(
        (inherited_manage_options[0],)+
        ({'label':'Silva /edit...', 'action':'edit'},)+
        inherited_manage_options[1:]
        )
    
    security.declareProtected(SilvaPermissions.View, 'is_cacheable')
    def is_cacheable(self):
        """Determine whether this object's rendering can be safely cached."""
        return True

InitializeClass(XMLDocumentBase)


class XMLDocumentVersionBase(CatalogedVersion):
    security = ClassSecurityInfo()

    __implements__ = IVersion

    manage_options = (
        {'label':'Edit',       'action':'manage_main'},
        ) + CatalogedVersion.manage_options
        
    # display edit screen as main management screen
    security.declareProtected('View management screens', 'manage_main')
    manage_main = PageTemplateFile('www/documentVersionEdit', globals())

    def __init__(self, id, title, iface_id):
        CatalogedVersion.inheritedAttribute('__init__')(self, id, title)
        self._xml_source = xmlsource.XMLSource()
        if iface_id is not None:
            self.add_interface(iface_id)
        self._transformer_id = None
        
    # MANIPULATORS
    security.declareProtected(SilvaPermissions.ChangeSilvaContent,
                              'set_xml_content')
    def set_xml_content(self, xml):
        """Expect UTF-8 encoded XML.
        """
        self._xml_source.set(xml)

    security.declareProtected(SilvaPermissions.ChangeSilvaContent,
                              'set_selected_xmlsource_id')
    def set_selected_xmlsource_id(self, iface_id):
        # remove old interface, if necessary
        old_iface_id = self.get_selected_xmlsource_id()
        if old_iface_id is not None:
            old_iface = xmlsource.resolvePythonPath(old_iface_id)
            directlyProvides(self._xml_source,
                             directlyProvidedBy(self._xml_source) - old_iface)
        if iface_id is None:
            return
        # add new interface
        new_iface = xmlsource.resolvePythonPath(iface_id)
        assert IInterface.providedBy(new_iface)
        assert new_iface.isOrExtends(interfaces.IXMLSource)
        directlyProvides(self._xml_source, new_iface)

    security.declareProtected(SilvaPermissions.ChangeSilvaContent,
                              'set_selected_transformer_id')
    def set_selected_transformer_id(self, transformer_id):
        assert (transformer_id is None or
                transformer_id in self.get_transformer_ids())
        self._transformer_id = transformer_id

    # ACCESSORS
    
    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'get_xml_content')
    def get_xml_content(self):
        """Returns UTF-8 encoded XML content, or None if no content yet.
        """
        return self._xml_source.get()

    security.declareProtected(SilvaPermissions.ReadSilvaContent,
                              'get_xmlsource_ids')
    def get_xmlsource_ids(self):
        """Get a list of XML source ids.

        Result will be sorted.
        """
        result = [id for id, iface in xmlsource.getXMLSourceInterfaces()]
        result.sort()
        return result
    
    security.declareProtected(SilvaPermissions.ReadSilvaContent,
                             'get_selected_xmlsource_id')
    def get_selected_xmlsource_id(self):
        """Get the interface id for the XML source selected.

        Returns None if no interface is yet selected.
        """
        xmlsource_ids = self.get_xmlsource_ids()
        for iface in directlyProvidedBy(self._xml_source):
            iface_id = xmlsource.getPythonPath(iface)
            if iface_id in xmlsource_ids:
                return iface_id
        return None
                
    security.declareProtected(SilvaPermissions.ReadSilvaContent,
                              'get_transformer_ids')
    def get_transformer_ids(self):
        """Get a list of transformer ids.

        Result will be sorted.
        """
        result = [id for id, iface in
                  xmlsource.getTransformers(self._xml_source)]
        result.sort()
        return result
    
    security.declareProtected(SilvaPermissions.ReadSilvaContent,
                              'get_selected_transformer_id')
    def get_selected_transformer_id(self):
        return self._transformer_id

    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'get_transformed')
    def get_transformed(self):
        transformer_id = self._transformer_id
        if transformer_id not in self.get_transformer_ids():
            raise error.TransformationError
        transformer = xmlsource.resolvePythonPath(transformer_id)
        # XXX encoding issues?
        return transformer(self._xml_source).transformToString()
    
    security.declareProtected(SilvaPermissions.AccessContentsInformation,
                              'fulltext')
    def fulltext(self):
        """Return the content of this object without any xml"""
        # XXX implement this
        pass
    
InitializeClass(XMLDocumentVersionBase)

def addHelper(container, factory, version_factory, id, title, iface_id):
    if not mangle.Id(container, id).isValid():
        return
    obj = factory(id)
    container._setObject(id, obj)
    obj = getattr(container, id)
    addVersionHelper(obj, version_factory, '0', title, iface_id)
    obj.create_version('0', None, None)
    return obj

def addVersionHelper(content, factory, nr, title, iface_id):
    version = factory(nr, title, iface_id)
    content._setObject(nr, version)
    version = content._getOb(nr)
    version.set_title(title)
    return version
