# python
from StringIO import StringIO
import os

# lxml
from lxml import etree

# Zope 2
from Persistence import Persistent

# Zope 3
from zope.interface import implements, providedBy
from zope.app.component.interface import searchInterfaceUtilities
# XXX what a weird dependency..
from zope.app.apidoc.utilities import getPythonPath

# SilvaFlexibleXML
from interfaces import IXMLSource, IXHTMLSource, IXSLTSource, IValidator, ITransformer
from error import ValidationError

class XMLSource(Persistent):
    implements(IXMLSource)
    
    def __init__(self, xml=None):
        self.set(xml)

    def set(self, xml):
        if xml is None:
            self._xml = None
            return
        try:
            tree = etree.parse(StringIO(xml))
        except SyntaxError:
            raise ValidationError, "Not well-formed XML"
        self._validate(tree)
        self._xml = xml

    def setTree(self, tree):
        self._validate(tree)
        f = StringIO()
        tree.write(f)
        xml = f.getvalue()
        f.close()
        self._xml = xml

    def _validate(self, tree):
        validator = IValidator(self)
        if not validator.validate(tree):
            raise ValidationError
        
    def get(self):
        return self._xml

    def getTree(self):
        return etree.parse(StringIO(self._xml))

def getXMLSourceInterfaces():
    """Get all known XML source interfaces.
    """
    return searchInterfaceUtilities(None, base=IXMLSource)

def getTransformers(obj):
    """Get all the registered transformers for an object.
    """
    from zope.component import getService
    service = getService('Adapters')
    transformers = {}
    for iface in providedBy(obj):
        if not iface.isOrExtends(IXMLSource):
            continue
        for transformer in service.subscriptions((iface,), ITransformer):
            transformers[getPythonPath(transformer)] = transformer
    return transformers.items()

def getPythonPaths(classes_or_interfaces):
    return [getPythonPath(o) for o in classes_or_interfaces]

def resolvePythonPath(path):
    parts = path.split('.')
    mod = __import__('.'.join(parts[:-1]), {}, {}, ('*',))
    return getattr(mod, parts[-1])

def localpath(path):
    return os.path.join(os.path.split(__file__)[0], path)

class NullValidator:
    def __init__(self, context):
        pass

    def validate(self, tree):
        return True


def cached(cache, key, factory):
    value = cache.get(key)
    if value is None:
        value = factory()
        cache[key] = value
    return value

relax_ng_cache = {}

class RelaxNGValidator:
    implements(IValidator)

    path = None

    def __init__(self, context):
        self._schema = cached(relax_ng_cache, self.path, self._factory)

    def _factory(self):
        f = open(self.path, 'r')
        schema = etree.RelaxNG(file=f)
        f.close()
        return schema
    
    def validate(self, tree):
        return self._schema.validate(tree)

class XSLTValidator(RelaxNGValidator):
    path = localpath('schema/xslt.rng')

class XHTMLValidator(RelaxNGValidator):
    path = localpath('schema/xhtml/xhtml.rng')

xslt_cache = {}

class XSLTTransformer:
    implements(ITransformer)

    path = None
    
    def __init__(self, context):
        self.context = context
        self._xslt = cached(xslt_cache, self.path, self._factory)

    def _factory(self):
        tree = etree.parse(self.path)
        return etree.XSLT(tree)
        
    def transform(self):
        return self._xslt.apply(self.context.getTree())
        
    def save(self, tree):
        return self._xslt.tostring(tree)
    
    def transformToString(self):
        tree = self.transform()
        return self.save(tree)

class FeatureSheetTransformer(XSLTTransformer):
    path = localpath('xslt/featuresheet.xsl')

class AnotherTransformer(XSLTTransformer):
    path = localpath('xslt/featuresheet.xsl')

class IndentTransformer(XSLTTransformer):
    path = localpath('xslt/indent.xslt')
    
